package org.firstinspires.ftc.teamcode.runmodes;

import com.qualcomm.robotcore.hardware.Gamepad;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.states.FieldStates;
import org.firstinspires.ftc.teamcode.subsystem.ArmSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.ClawSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.LiftSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.MotorsSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.WristSubsystem;
import org.firstinspires.ftc.teamcode.util.action.Actions;
import org.firstinspires.ftc.teamcode.util.action.SequentialAction;

public class DevTeleopRunMode {

    /*
    Subsystems
     */
    private MotorsSubsystem motors;
    public ClawSubsystem claw;
    public ArmSubsystem arm;
    public WristSubsystem wrist;
    public LiftSubsystem lift;

    /*
    Controllers
     */
    public Gamepad GP1;
    public Gamepad GP2;
    public Gamepad currentGP1;
    public Gamepad previousGP1;
    public Gamepad currentGP2;
    public Gamepad previousGP2;
    public FieldStates fieldStates;

    public DevTeleopRunMode(HardwareMap hardwareMap, Telemetry telemetry, Gamepad gp1, Gamepad gp2) {
        this.motors = new MotorsSubsystem(hardwareMap, telemetry);
        this.claw = new ClawSubsystem(hardwareMap, telemetry);
        this.arm = new ArmSubsystem(hardwareMap, telemetry);
        this.wrist = new WristSubsystem(hardwareMap, telemetry);
        this.lift = new LiftSubsystem(hardwareMap, telemetry);
        this.GP1 = gp1;
        this.GP2 = gp2;
        this.currentGP1 = new Gamepad();
        this.currentGP2 = new Gamepad();
        this.previousGP1 = new Gamepad();
        this.previousGP2 = new Gamepad();
        this.fieldStates = new FieldStates();
    }

    public void init() {
        this.motors.init();
        this.claw.init();
        this.arm.init();
        this.wrist.init();
        this.lift.init();
    }

    public void update() {
        this.previousGP1.copy(currentGP1);
        this.currentGP1.copy(this.GP1);
        this.previousGP2.copy(currentGP2);
        this.currentGP2.copy(this.GP2);
        this.theDrop();
        this.theLift();
        this.thePickup();
        this.theTravel();
        this.theLowBucketScore();
        this.theHighBucketScore();
    }

    /*
    Controller: 1
    Button: A
    Action: On button press, Arm hovers the floor with wrist parallel to arm
     */
    public void theDrop() {
        if (this.currentGP1.a && !this.previousGP1.a) {
            if (fieldStates.getFieldLocation() == FieldStates.FieldLocation.FIELD) {
                Actions.runBlocking(new SequentialAction(
                        this.arm.toFloorPosition,
                        this.wrist.toFloorPosition
                ));
            }
        }
    }

    /*
    Controller: 1
    Button: Right Bumper
    Action: On button press, open and closes claw
     */
    public void thePickup() {
        if (this.currentGP1.right_bumper && !this.previousGP1.right_bumper) {
            this.claw.switchState();
        }
    }

    /*
    Controller: 1
    Button: B
    Action: On button press, lift arm and prepare wrist for bucket
     */
    public void theLift() {
        if (this.currentGP1.b && !this.previousGP1.b) {
            Actions.runBlocking(new SequentialAction(
                    this.arm.toParkPosition,
                    this.wrist.toBucketPosition
            ));
        }
    }

    /*
    Controller: 2
    Button: Y
    Action: On button press, lift to low bucket height,
            arm to bucket position, wrist to bucket position
     */
    public void theLowBucketScore() {
        if (this.currentGP2.y && !this.previousGP2.y) {
            fieldStates.setFieldLocation(FieldStates.FieldLocation.BUCKET);
            Actions.runBlocking(new SequentialAction(
                    lift.toLowBucket,
                    arm.toBucketPosition,
                    wrist.toBucketPosition
            ));
        }
    }

    /*
    Controller: 2
    Button: A
    Action: On button press, lift to low bucket height,
            arm to bucket position, wrist to bucket position
     */
    public void theHighBucketScore() {
        if (this.currentGP2.a && !this.previousGP2.a) {
            fieldStates.setFieldLocation(FieldStates.FieldLocation.BUCKET);
            Actions.runBlocking(new SequentialAction(
                    lift.toHighBucket,
                    arm.toBucketPosition,
                    wrist.toBucketPosition
            ));
        }
    }

    /*
    Controller: 2
    Button: Direction Pad DOWN
    Action: On directional press, lift to floor height,
            arm to bucket position, wrist to floor position
     */
    public void theTravel(){
        if (this.currentGP2.dpad_down && !this.previousGP2.dpad_down){
            fieldStates.setFieldLocation(FieldStates.FieldLocation.FIELD);
            Actions.runBlocking(new SequentialAction(
                    lift.toFloor,
                    arm.toBucketPosition,
                    wrist.toFloorPosition
            ));
        }
    }

}
